@@ -1,14 +1,15 @@
 """
-Python generator functions for the various wxSizerS
+Python generator functions for the various wxSizers
 
 @copyright: 2002-2007 Alberto Griggio
 @copyright: 2013-2016 Carsten Grohmann
+@copyright: 2019 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
 
 import common
-from .edit_sizers import BaseSizerBuilder
+from .edit_sizers import BaseSizerBuilder, SlotGenerator
 
 
 class BasePythonSizerBuilder(BaseSizerBuilder):
@@ -18,64 +19,49 @@ class BasePythonSizerBuilder(BaseSizerBuilder):
 
     tmpl_SetSizer = '%(parent_widget)s.SetSizer(%(sizer_name)s)\n'
     tmpl_Fit = '%(sizer_name)s.Fit(%(parent_widget)s)\n'
+    tmpl_Realize = '%(sizer_name)s.Realize()\n'
     tmpl_SetSizeHints = '%(sizer_name)s.SetSizeHints(%(parent_widget)s)\n'
 
     def _get_wparent(self, obj):
-        while obj.is_sizer:
-            obj = obj.node.parent.widget
-        if not obj.is_toplevel:
-            parent = 'self.%s' % obj.name
-        else:
-            parent = 'self'
-        return parent
-
+        window = obj.parent_window
+        if window.IS_CLASS:
+            return 'self'
+        return 'self.%s' % window.name
 
 
 class PythonBoxSizerBuilder(BasePythonSizerBuilder):
-    klass = 'wxBoxSizer'
     tmpl = '%(sizer_name)s = %(klass)s(%(orient)s)\n'
 
 
+class PythonStdDialogButtonSizerBuilder(BasePythonSizerBuilder):
+    tmpl = '%(sizer_name)s = %(klass)s()\n'
+
+
 class PythonWrapSizerBuilder(PythonBoxSizerBuilder):
-    klass = 'wxWrapSizer'
+    pass
 
 
 class PythonStaticBoxSizerBuilder(BasePythonSizerBuilder):
-    klass = 'wxStaticBoxSizer'
     tmpl = '%(sizer_name)s = %(klass)s(wx.StaticBox(%(parent_widget)s, %(wxIDANY)s, %(label)s), %(orient)s)\n'
 
 
-
 class PythonGridSizerBuilder(BasePythonSizerBuilder):
-    klass = 'wxGridSizer'
     tmpl = '%(sizer_name)s = %(klass)s(%(rows)s, %(cols)s, %(vgap)s, %(hgap)s)\n'
 
 
-
 class PythonFlexGridSizerBuilder(PythonGridSizerBuilder):
-    klass = 'wxFlexGridSizer'
-
     tmpl_AddGrowableRow = '%(sizer_name)s.AddGrowableRow(%(row)s)\n'
     tmpl_AddGrowableCol = '%(sizer_name)s.AddGrowableCol(%(col)s)\n'
 
 
 class PythonGridBagSizerBuilder(PythonFlexGridSizerBuilder):
-    klass = 'wxGridBagSizer'
     tmpl = '%(sizer_name)s = %(klass)s(%(vgap)s, %(hgap)s)\n'
 
 
-import wcodegen
-
-class PythonSizerSlotGenerator(wcodegen.PythonWidgetCodeWriter):
-    # spacers and empty sizer slots are generally handled by a hack:
-    # The the implementations of add_sizeritem() contains more details.
-    # The code generation code is already implemented in base class.
-    pass
-
-
 def initialize():
     cn = common.class_names
     cn['EditBoxSizer'] = 'wxBoxSizer'
+    cn['EditStdDialogButtonSizer'] = 'wxStdDialogButtonSizer'
     cn['EditWrapSizer'] = 'wxWrapSizer'
     cn['EditStaticBoxSizer'] = 'wxStaticBoxSizer'
     cn['EditGridSizer'] = 'wxGridSizer'
@@ -84,14 +70,13 @@ def initialize():
 
     pygen = common.code_writers.get("python")
     if pygen:
-        awh = pygen.add_widget_handler
+        awh = pygen.register_widget_code_generator
         awh('wxBoxSizer', PythonBoxSizerBuilder())
+        awh('wxStdDialogButtonSizer', PythonStdDialogButtonSizerBuilder())
         awh('wxWrapSizer', PythonWrapSizerBuilder())
         awh('wxStaticBoxSizer', PythonStaticBoxSizerBuilder())
         awh('wxGridSizer', PythonGridSizerBuilder())
         awh('wxFlexGridSizer', PythonFlexGridSizerBuilder())
         awh('wxGridBagSizer', PythonGridBagSizerBuilder())
 
-    # handle SizerSlot
-    #common.class_names['EditSpacer'] = klass
-    common.register('python', "sizerslot", PythonSizerSlotGenerator("sizerslot"))
+    common.register('python', "sizerslot", SlotGenerator("python"))
