@@ -5,24 +5,24 @@ This module provides a own logger class as well as specific functions to improve
 
 wxGlade uses the python logging instance with three log handler attached.
 
-The first handler L{StringHandler} is used to cache messages later displaying calling getBufferAsList() or
+The first handler StringHandler is used to cache messages later displaying calling getBufferAsList() or
 getBufferAsString().
 
-The second handler C{logging.StreamHandler} to print error messages to sys.stderr.
+The second handler logging.StreamHandler to print error messages to sys.stderr.
 
-The third handler C{logging.FileHandler} writes all messages into a file. This
+The third handler logging.FileHandler writes all messages into a file. This
 behaviour is useful to store logged exceptions permanently.
 
 @todo: Integrate Unicode logging fix.
 
 @copyright: 2013-2016 Carsten Grohmann
-@copyright: 2017 Dietmar Schwertberger
+@copyright: 2017-2020 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
-import datetime, inspect
+import datetime
 import logging, logging.handlers
-import os, sys, types, traceback
+import os, sys, traceback
 
 try:
     _nameToLevel = logging._levelNames
@@ -181,7 +181,10 @@ def init(filename='wxglade.log', encoding='utf-8', level=None):
 
     # instantiate console handler
     console_logger = logging.StreamHandler()
-    console_logger.setLevel(logging.INFO)
+    if config.debugging:
+        console_logger.setLevel(logging.DEBUG)
+    else:
+        console_logger.setLevel(logging.INFO)
     console_logger.setFormatter(default_formatter)
     logger.addHandler(console_logger)
 
@@ -294,15 +297,12 @@ def getMessage(self):
 
     Return the message for this LogRecord after merging any user-supplied arguments with the message.
     This specific version tries to handle Unicode user-supplied arguments."""
-    if not hasattr(types, "UnicodeType"):  # if no unicode support... # XXX
-        msg = str(self.msg)
-    else:
-        msg = self.msg
-        if not isinstance(msg, types.StringTypes):
-            try:
-                msg = str(self.msg)
-            except UnicodeError:
-                msg = self.msg      # Defer encoding till later
+    msg = self.msg
+    if not isinstance(msg, compat.basestring):
+        try:
+            msg = compat.unicode(msg)
+        except UnicodeError:
+            msg = self.msg      # Defer encoding till later
     if self.args:
         try:
             msg = msg % self.args
