@@ -1,16 +1,15 @@
 """\
 wxRadioButton widget configuration
 
-@copyright: 2014 Carsten Grohmann
-@license: MIT (see license.txt) - THIS PROGRAM COMES WITH NO WARRANTY
+@copyright: 2014-2016 Carsten Grohmann
+@license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
 config = {
     'wxklass': 'wxRadioButton',
     'style_defs': {
         'wxRB_GROUP': {
-            'desc': _('Marks the beginning of a new group of radio '
-                      'buttons.'),
+            'desc': _('Marks the beginning of a new group of radio buttons.'),
         },
         'wxRB_SINGLE': {
             'desc': _('In some circumstances, radio buttons that are not '
@@ -19,14 +18,11 @@ config = {
                       'button as not belonging to a group, and implement '
                       'the mutually-exclusive group behaviour yourself.'),
         },
-        'wxRB_USE_CHECKBOX': {
-            'desc': _('Use a checkbox button instead of radio button ('
-                      'currently supported only on PalmOS).'),
-            'supported_by': ('wx2',),
-        },
     },
-    'box_label': _('Style'),
-    'style_list': ['wxRB_GROUP', 'wxRB_SINGLE', 'wxRB_USE_CHECKBOX'],
+    'style_list': ['wxRB_GROUP', 'wxRB_SINGLE'],
+    'events': {
+        'EVT_RADIOBUTTON': {},
+    },
 }
 
 
