@@ -3,24 +3,25 @@ Spacers to use in sizers
 
 @copyright: 2002-2007 Alberto Griggio
 @copyright: 2014-2016 Carsten Grohmann
-@copyright: 2016-2019 Dietmar Schwertberger
+@copyright: 2016-2020 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
 import wx
 import common, misc
-from tree import Node
 import new_properties as np
 from edit_windows import ManagedBase
 
 
 class EditSpacer(ManagedBase):
     "Class to handle spacers for sizers"
-    _PROPERTIES = ["Layout", "width", "height", "pos", "proportion", "border", "flag"]
+    WX_CLASS = 'spacer'
+    IS_NAMED = False
+    _PROPERTIES = ["Layout", "width", "height", "proportion", "border", "flag"]
     PROPERTIES = _PROPERTIES + ManagedBase.EXTRA_PROPERTIES
 
-    def __init__(self, name, parent, id, width, height, sizer, pos):
-        ManagedBase.__init__(self, name, 'spacer', parent, id, sizer, pos)
+    def __init__(self, parent, index, width, height):
+        ManagedBase.__init__(self, 'spacer', parent, index)
 
         # initialise instance properties
         self.width  = np.SpinProperty(width,  immediate=True)
@@ -28,7 +29,7 @@ class EditSpacer(ManagedBase):
 
     def create_widget(self):
         style = wx.SIMPLE_BORDER | wx.FULL_REPAINT_ON_RESIZE
-        self.widget = wx.Window(self.parent.widget, self.id, size=(self.width, self.height), style=style)
+        self.widget = wx.Window(self.parent_window.widget, wx.ID_ANY, size=(self.width, self.height), style=style)
         self.widget.GetBestSize = self.widget.GetSize
         self.widget.Bind(wx.EVT_PAINT, self.on_paint)
 
@@ -50,18 +51,18 @@ class EditSpacer(ManagedBase):
         dc.DrawRectangle(x-1, y-1, tw+2, th+2)
         dc.DrawText(text, x, y)
 
-    def properties_changed(self, modified):
+    def _properties_changed(self, modified, actions):
         if not modified or "width" in modified or "height" in modified:
             size = (self.width, self.height)
             if self.widget: self.widget.SetSize(size)
-            self.sizer.set_item_best_size(self, size=size)
-        ManagedBase.properties_changed(self, modified)
+            self.parent.set_item_best_size(self, size=size)
+            actions.add("layout")
+        ManagedBase._properties_changed(self, modified, actions)
 
 
 class _Dialog(wx.Dialog):
-    def __init__(self, parent):
-        pos = wx.GetMousePosition()
-        wx.Dialog.__init__(self, misc.get_toplevel_parent(parent), -1, _("Enter size"), pos)
+    def __init__(self):
+        wx.Dialog.__init__(self, common.main, -1, _("Enter size"), wx.GetMousePosition())
         # the controls
         self.width  = wx.SpinCtrl(self, -1, "20")
         self.height = wx.SpinCtrl(self, -1, "20")
@@ -82,16 +83,15 @@ class _Dialog(wx.Dialog):
         btn = wx.Button(self, wx.ID_OK, _('OK') )
         btn.SetDefault()
         hsizer.Add(btn, 1, wx.ALL, 5)
-        sizer.Add(hsizer, 0, wx.EXPAND|wx.ALIGN_CENTER )
+        sizer.Add(hsizer, 0, wx.EXPAND)
 
         self.SetAutoLayout(True)
         self.SetSizer(sizer)
         sizer.Fit(self)
 
-
-def builder(parent, sizer, pos):
+def builder(parent, index):
     "factory function for EditSpacer objects"
-    dialog = _Dialog(parent)
+    dialog = _Dialog()
     with misc.disable_stay_on_top(common.adding_window or parent):
         res = dialog.ShowModal()
     width  = dialog.width.GetValue()
@@ -100,34 +100,21 @@ def builder(parent, sizer, pos):
     if res != wx.ID_OK:
         return
 
-    name = 'spacer'
     with parent.frozen():
-        spacer = EditSpacer( name, parent, wx.NewId(), width, height, sizer, pos )
-        node = Node(spacer)
-        spacer.node = node
-        if parent.widget: spacer.create()
-    common.app_tree.insert(node, sizer.node, pos-1)
+        editor = EditSpacer( parent, index, width, height )
+        if parent.widget: editor.create()
+    return editor
 
 
-def xml_builder(attrs, parent, sizer, sizeritem, pos=None):
+def xml_builder(parser, base, name, parent, index):
     "factory to build EditSpacer objects from a XML file"
-    from xml_parse import XmlParsingError
-    if not sizer or not sizeritem:
-        raise XmlParsingError(_("sizer or sizeritem object cannot be None"))
-    spacer = EditSpacer('spacer', parent, wx.NewId(), 1, 1, sizer, pos)
-    #sizer.set_item(spacer.pos, proportion=sizeritem.proportion, span=sizeritem.span, flag=sizeritem.flag, border=sizeritem.border)
-    node = Node(spacer)
-    spacer.node = node
-    if pos is None:
-        common.app_tree.add(node, sizer.node)
-    else:
-        common.app_tree.insert(node, sizer.node, pos-1)
-    return spacer
+    return EditSpacer(parent, index, 1, 1)
 
 
 def initialize():
     "initialization function for the module: returns a wx.BitmapButton to be added to the main palette"
+    common.widget_classes['EditSpacer'] = EditSpacer
     common.widgets['EditSpacer'] = builder
     common.widgets_from_xml['EditSpacer'] = xml_builder
 
-    return common.make_object_button('EditSpacer', 'spacer.xpm')
+    return common.make_object_button('EditSpacer', 'spacer.png')
