@@ -2,8 +2,8 @@
 Code generator functions for wxStaticLine objects
 
 @copyright: 2002-2007 Alberto Griggio
-@copyright: 2014 Carsten Grohmann
-@license: MIT (see license.txt) - THIS PROGRAM COMES WITH NO WARRANTY
+@copyright: 2014-2016 Carsten Grohmann
+@license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
 import common
@@ -13,26 +13,22 @@ import wcodegen
 class PythonStaticLineGenerator(wcodegen.PythonWidgetCodeWriter):
     tmpl = '%(name)s = %(klass)s(%(parent)s, %(id)s%(style)s)\n'
 
-# end of class PythonStaticLineGenerator
 
 
 class CppStaticLineGenerator(wcodegen.CppWidgetCodeWriter):
-    extra_headers = ['<wx/statline.h>']
+    import_modules = ['<wx/statline.h>']
     tmpl = '%(name)s = new %(klass)s(%(parent)s, %(id)s%(style)s);\n'
 
-# end of class CppStaticLineGenerator
 
 
 def xrc_code_generator(obj):
     xrcgen = common.code_writers['XRC']
 
     class XrcCodeGenerator(xrcgen.DefaultXrcObject):
-        def write(self, *args, **kwds):
-            try:
-                del self.properties['attribute']
-            except KeyError:
-                pass
-            xrcgen.DefaultXrcObject.write(self, *args, **kwds)
+        def write(self, out_file, ntabs):
+            style = self.widget.properties["style"].get_string_value()
+            properties = {"attribute":None, "style":style}
+            xrcgen.DefaultXrcObject.write(self, out_file, ntabs, properties)
 
     return XrcCodeGenerator(obj)
 
@@ -41,5 +37,5 @@ def initialize():
     klass = 'wxStaticLine'
     common.class_names['EditStaticLine'] = klass
     common.register('python', klass, PythonStaticLineGenerator(klass))
-    common.register('C++', klass, CppStaticLineGenerator(klass))
-    common.register('XRC', klass, xrc_code_generator)
+    common.register('C++',    klass, CppStaticLineGenerator(klass))
+    common.register('XRC',    klass, xrc_code_generator)
