@@ -3,13 +3,12 @@ wxFrame and wxStatusBar objects
 
 @copyright: 2002-2007 Alberto Griggio
 @copyright: 2014-2016 Carsten Grohmann
-@copyright: 2016-2019 Dietmar Schwertberger
+@copyright: 2016-2020 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
 import wx
 import common, misc
-from tree import Node, WidgetTree
 from wcodegen.taghandler import BaseXmlBuilderTagHandler
 import new_properties as np
 from edit_windows import EditBase, EditStylesMixin
@@ -35,8 +34,7 @@ class FieldsHandler(BaseXmlBuilderTagHandler):
             char_data = self.get_char_data()
             self.fields.append([char_data, self.width])
         else:  # name == 'fields'
-            self.owner.properties["fields"].set(self.fields)
-            self.owner.properties_changed(["fields"])
+            self.owner.properties["fields"].load(self.fields)
             return True
 
 
@@ -58,11 +56,13 @@ class EditStatusBar(EditBase, EditStylesMixin):
     _hidden_frame = None
     update_widget_style = False  # updating does not seem to have an effect
 
+    WX_CLASS = 'wxStatusBar'
     _PROPERTIES = ["Widget", "style", "fields"]
     PROPERTIES = EditBase.PROPERTIES + _PROPERTIES + EditBase.EXTRA_PROPERTIES
+    CHILDREN = 0
 
-    def __init__(self, name, klass, parent):
-        EditBase.__init__( self, name, klass, parent, wx.NewId(), custom_class=False )
+    def __init__(self, name, parent):
+        EditBase.__init__( self, name, parent, index="_statusbar")
         EditStylesMixin.__init__(self)
 
         # for the statusbar fields
@@ -70,9 +70,6 @@ class EditStatusBar(EditBase, EditStylesMixin):
         self.fields = FieldsProperty(fields)
         self.window_id = None  # just a dummy for code generation
 
-        self.node = Node(self)
-        common.app_tree.add(self.node, parent.node)
-
     def create_widget(self):
         self.widget = wx.StatusBar(self.parent.widget, -1)
         self.widget.Bind(wx.EVT_LEFT_DOWN, self.on_set_focus)
@@ -80,6 +77,10 @@ class EditStatusBar(EditBase, EditStylesMixin):
         if self.parent.widget:
             self.parent.widget.SetStatusBar(self.widget)
 
+    def remove(self):
+        EditBase.remove(self)
+        self.parent.properties['statusbar'].set(False)
+
     def _set_fields(self):
         if not self.widget: return
         fields = self.fields  # values is a list of lists
@@ -94,24 +95,12 @@ class EditStatusBar(EditBase, EditStylesMixin):
 
         self.widget.SetStatusWidths(widths)
 
-    def remove(self, *args, **kwds):
-        if not kwds.get('do_nothing', False):
-            self.parent.properties['statusbar'].set(False)
-            if self.parent.widget:
-                self.parent.widget.SetStatusBar(None)
-            try:
-                self.parent._statusbar = None
-            except KeyError:
-                pass
-            if self.widget:
-                self.widget.Hide()
-            EditBase.remove(self)
-        else:
-            if EditStatusBar._hidden_frame is None:
-                EditStatusBar._hidden_frame = wx.Frame(None, -1, "")
-            if self.widget is not None:
-                self.widget.Reparent(EditStatusBar._hidden_frame)
-            self.widget = None
+    def destroy_widget(self, level):
+        # if parent is being deleted, we rely on this being destroyed
+        if level==0 and not self.IS_TOPLEVEL and self.parent.widget:
+            self.parent.widget.SetStatusBar(None)
+        if level==0:
+            EditBase.destroy_widget(self, level)
 
     def popup_menu(self, *args):
         pass  # to avoid strange segfault :)
@@ -121,13 +110,13 @@ class EditStatusBar(EditBase, EditStylesMixin):
             return FieldsHandler(self)
         return None
 
-    def properties_changed(self, modified):
+    def _properties_changed(self, modified, actions):
         if not modified or "fields" in modified:
             self._set_fields()
-        EditStylesMixin.properties_changed(self, modified)
-        EditBase.properties_changed(self, modified)
+        EditStylesMixin._properties_changed(self, modified, actions)
+        EditBase._properties_changed(self, modified, actions)
 
-    def check_compatibility(self, widget, typename=None, report=False):
+    def check_compatibility(self, widget, typename=None):
         return (False,"No pasting possible here.")
     def check_drop_compatibility(self):
         return (False,"Edit fields in Properties -> Widget")
@@ -138,12 +127,12 @@ _NUMBER = 0
 class Dialog(wx.Dialog):
     def __init__(self):
         global _NUMBER
-        wx.Dialog.__init__(self, None, -1, _('Select toolbar class'))
+        wx.Dialog.__init__(self, None, -1, _('Select statusbar class'))
         
-        if common.app_tree.app.language.lower() == 'xrc':
-            klass = 'wxToolBar'
+        if common.root.language.lower() == 'xrc':
+            klass = 'wxStatusBar'
         else:
-            klass = 'MyToolBar%s' % (_NUMBER or "")
+            klass = 'MyStatusBar%s' % (_NUMBER or "")
             _NUMBER += 1
 
         # class
@@ -161,53 +150,26 @@ class Dialog(wx.Dialog):
         szr.Fit(self)
 
 
-def builder(parent, sizer, pos):
-    "factory function for EditToolBar objects"
 
-    dialog = Dialog()
-    with misc.disable_stay_on_top(common.adding_window or parent):
-        res = dialog.ShowModal()
-    klass = dialog.klass
-    dialog.Destroy()
-    if res != wx.ID_OK:
-        if number[0] > 0:
-            number[0] -= 1
-        return
-
-    name = 'statusbar_%d' % (number[0] or 1)
-    while common.app_tree.has_name(name):
-        number[0] += 1
-        name = 'statusbar_%d' % number[0]
-
-    with parent.frozen():
-        widget = EditStatusBar(name, klass, parent)
-        widget.node = Node(widget)
-        common.app_tree.add(widget.node)
-        if parent.widget: widget.create()
+def builder(parent, index, klass):
+    # only used from Frame.menubar property
+    editor = EditStatusBar(parent.name+"_statusbar", parent)
+    if parent.widget: editor.create()
+    return editor
 
 
-
-def xml_builder(attrs, parent, sizer, sizeritem, pos=None):
+def xml_builder(parser, base, name, parent, index):
     "factory to build EditStatusBar objects from a XML file"
-    name = attrs.get('name')
-    if parent:
-        if name:
-            parent._statusbar.properties["name"].set(name)
-            parent._statusbar.properties_changed(["name"])
-        return parent._statusbar
-    else:
-        widget = EditStatusBar(name, attrs.get('class', 'wxStatusBar'), None)
-        widget.node = Node(widget)
-        common.app_tree.add(widget.node)
-        return widget
+    parent.properties["statusbar"].set(True)
+    return EditStatusBar(name, parent)
 
 
 def initialize():
     "initialization function for the module: returns a wxBitmapButton to be added to the main palette."
+    common.widget_classes['EditStatusBar'] = EditStatusBar
+    common.widgets['EditStatusBar'] = builder   # only used from Frame.menubar property
     common.widgets_from_xml['EditStatusBar'] = xml_builder
-    common.widgets['EditStatusBar'] = builder
-    #return common.make_object_button('EditStatusBar', 'statusbar.xpm')
-    # no standalone status bar any more
     import config, os
-    WidgetTree.images['EditStatusBar'] = os.path.join(config.icons_path, 'statusbar.xpm')
+    from tree import WidgetTree
+    WidgetTree.images['EditStatusBar'] = os.path.join(config.icons_path, 'statusbar.png')
     return []
