@@ -2,8 +2,8 @@
 Lisp generator functions for wxTreeCtrl objects
 
 @copyright: 2002-2004 D. H. aka crazyinsomniac on sourceforge
-@copyright: 2014 Carsten Grohmann
-@license: MIT (see license.txt) - THIS PROGRAM COMES WITH NO WARRANTY
+@copyright: 2014-2016 Carsten Grohmann
+@license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
 import common
@@ -11,10 +11,7 @@ import wcodegen
 
 
 class LispTreeCtrlGenerator(wcodegen.LispWidgetCodeWriter):
-    tmpl = '(setf %(name)s (%(klass)s_Create %(parent)s %(id)s ' \
-           '-1 -1 -1 -1 %(style)s))\n'
-
-# end of class LispTreeCtrlGenerator
+    tmpl = '(setf %(name)s (%(klass)s_Create %(parent)s %(id)s -1 -1 -1 -1 %(style)s))\n'
 
 
 def initialize():
