@@ -4,7 +4,7 @@ Entry point of wxGlade
 
 @copyright: 2002-2007 Alberto Griggio
 @copyright: 2013-2016 Carsten Grohmann
-@copyright: 2016-2018 Dietmar Schwertberger
+@copyright: 2016-2020 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
@@ -44,7 +44,7 @@ def my_displayhook(value):
 sys.displayhook = my_displayhook
 
 
-import common, config, compat, log, errors
+import common, config, compat, log
 
 
 def parse_command_line():
@@ -58,7 +58,7 @@ def parse_command_line():
     version = _("wxGlade version %s\n"
                 "Copyright (C) 2007-2012 Alberto Griggio\n"
                 "Copyright (C) 2011-2016 Carsten Grohmann\n"
-                "Copyright (C) 2016-2017 Dietmar Schwertberger\n"
+                "Copyright (C) 2016-2020 Dietmar Schwertberger\n"
                 "License MIT: The MIT License\n"
                 "             <http://www.opensource.org/licenses/mit-license.php>") % config.get_version()
     usage = _("Usage: wxglade <WXG File>             start the wxGlade GUI\n"
@@ -94,6 +94,7 @@ def parse_command_line():
     # from '.' if it is invoked from a shell script).
     if len(args) == 1:
         filename = args[0]
+        if filename.startswith("file://"): filename = filename[7:]
         if not os.path.isabs(filename):
             filename = os.path.join(os.getcwd(), filename)
         filename = os.path.normpath(os.path.expanduser(filename))
@@ -139,7 +140,7 @@ def command_line_code_generation(filename, language, out_path=None):
 
     try:
         if language not in common.code_writers:
-            raise errors.WxgMissingCodeWriter(language)
+            raise ValueError('Code writer for "%s" is not available.'%language)
 
         writer = common.code_writers[language]
         CodeWriter( writer=writer, input=filename, out_path=out_path )
@@ -160,14 +161,14 @@ def _guiless_open_app(filename):
     Returns True if successful."""
     import time
     from xml.sax import SAXParseException
-    from xml_parse import XmlWidgetBuilder, ProgressXmlWidgetBuilder, XmlParsingError
+    from xml_parse import XmlWidgetBuilder, XmlParsingError
     error_msg = None
     infile = None
 
     start = time.time()
 
-    common.app_tree.clear()
-    common.app_tree.app.init()
+    common.root.clear()
+    common.root.init()
 
     try:
         try:
@@ -175,7 +176,7 @@ def _guiless_open_app(filename):
             input_file_version = None
 
             if not isinstance(filename, list):
-                common.app_tree.app.filename = filename
+                common.root.filename = filename
                 # decoding will done automatically by SAX XML library
                 if compat.PYTHON2:
                     infile = open(filename)
@@ -184,7 +185,7 @@ def _guiless_open_app(filename):
                 if hasattr(infile, "seek"):
                     # try to read file version number from the first few lines
                     import re
-                    version_re = re.compile("<!-- generated by wxGlade (\d+)\.(\d+)\.(\d+)(\S*)\s*")
+                    version_re = re.compile(r"<!-- generated by wxGlade (\d+)\.(\d+)\.(\d+)(\S*)\s*")
                     for n in range(3):
                         match = version_re.match( infile.readline() )
                         if match:
@@ -194,7 +195,7 @@ def _guiless_open_app(filename):
                     infile.seek(0)
 
             else:
-                common.app_tree.app.filename = None
+                common.root.filename = None
 
             p = XmlWidgetBuilder(filename, input_file_version)
 
@@ -222,30 +223,30 @@ def _guiless_open_app(filename):
             infile.close()
 
         if error_msg:
-            common.app_tree.clear()
-            common.app_tree.app.new()
-            common.app_tree.app.saved = True
+            common.root.clear()
+            common.root.new()
+            common.root.saved = True
 
             logging.error(error_msg)
 
             return False
 
-    if common.app_tree.app.is_template:
+    if common.root.is_template:
         logging.info(_("Template loaded"))
-        common.app_tree.app.template_data = template.Template(filename)
-        common.app_tree.app.filename = None
+        common.root.template_data = template.Template(filename)
+        common.root.filename = None
 
     end = time.time()
-    logging.info(_('Loading time: %.5f'), end - start)
+    logging.debug(_('Loading time: %.5f'), end - start)
 
-    common.app_tree.app.saved = True
+    common.root.saved = True
     #common.property_panel.Raise()
 
     duration = end - start
     if filename:
-        logging.info( _("Loaded %s in %.2f seconds") % (os.path.basename(filename), duration ))
+        logging.debug( _("Loaded %s in %.2f seconds") % (os.path.basename(filename), duration ))
     else:
-        logging.info( _("Loaded in %.2f seconds") % duration )
+        logging.debug( _("Loaded in %.2f seconds") % duration )
 
     return True
 
@@ -263,11 +264,7 @@ def command_line_code_generation(filename, language, out_path=None):
     # following lines are taken from main.wxGlade().OnInit() and
     # main.wxGladeFrame.__init__()
     common.init_preferences()
-    app = application.Application()
-    # The following lines contain code from tree.WidgetTree.__init__()
-    root_node = tree.Node(app)
-    app.node = root_node
-    common.app_tree = tree.Tree(root_node, app)
+    common.root = app = application.Application()
 
     # Now we can load the file
     if filename is not None:
@@ -276,18 +273,21 @@ def command_line_code_generation(filename, language, out_path=None):
             sys.exit(1)
     try:
         if language not in common.code_writers:
-            raise errors.WxgMissingCodeWriter(language)
-        common.app_tree.app.properties["language"].set(language)
-        common.app_tree.app.generate_code(out_path=out_path)
-    except errors.WxgBaseException as inst:
-        logging.error(inst)
-        sys.exit(inst)
+            raise ValueError('Code writer for "%s" is not available.'%language)
+        common.root.properties["language"].set(language)
+        common.root.generate_code(out_path=out_path)
+    #except errors.WxgBaseException as inst:
+        #if config.debugging: raise
+        #logging.error(inst)
+        #sys.exit(inst)
     except Exception:
+        if config.debugging: raise
         logging.error( _("An exception occurred while generating the code for the application.\n"
                          "If you think this is a wxGlade bug, please report it.") )
         logging.exception(_('Internal Error'))
         sys.exit(1)
-    sys.exit(0)
+    if not config.testing:
+        sys.exit(0)
 
 
 def init_stage1(options):
@@ -297,27 +297,28 @@ def init_stage1(options):
     common.init_paths(options)
 
     # initialise own logging extensions
-    log.init(filename=config.log_file, encoding='utf-8', level='INFO')
+    log.init(filename=config.log_file, encoding='utf-8',
+             level='DEBUG' if config.debugging else 'INFO')
     atexit.register(log.deinit)
 
     # print versions
     logging.info( _('Starting wxGlade version "%s" on Python %s'), config.version, config.py_version )
 
     # print used paths
-    logging.info(_('Base directory:             %s'), config.wxglade_path)
-    logging.info(_('Documentation directory:    %s'), config.docs_path)
-    logging.info(_('Icons directory:            %s'), config.icons_path)
-    logging.info(_('Build-in widgets directory: %s'), config.widgets_path)
-    logging.info(_('Template directory:         %s'), config.templates_path)
-    logging.info(_('Credits file:               %s'), config.credits_file)
-    logging.info(_('License file:               %s'), config.license_file)
-    logging.info(_('Manual file:                %s'), config.manual_file)
-    logging.info(_('Tutorial file:              %s'), config.tutorial_file)
-    logging.info(_('Home directory:             %s'), config.home_path)
-    logging.info(_('Application data directory: %s'), config.appdata_path)
-    logging.info(_('Configuration file:         %s'), config.rc_file)
-    logging.info(_('History file:               %s'), config.history_file)
-    logging.info(_('Log file:                   %s'), config.log_file)
+    logging.debug(_('Base directory:             %s'), config.wxglade_path)
+    logging.debug(_('Documentation directory:    %s'), config.docs_path)
+    logging.debug(_('Icons directory:            %s'), config.icons_path)
+    logging.debug(_('Build-in widgets directory: %s'), config.widgets_path)
+    logging.debug(_('Template directory:         %s'), config.templates_path)
+    logging.debug(_('Credits file:               %s'), config.credits_file)
+    logging.debug(_('License file:               %s'), config.license_file)
+    logging.debug(_('Manual file:                %s'), config.manual_file)
+    logging.debug(_('Tutorial file:              %s'), config.tutorial_file)
+    logging.debug(_('Home directory:             %s'), config.home_path)
+    logging.debug(_('Application data directory: %s'), config.appdata_path)
+    logging.debug(_('Configuration file:         %s'), config.rc_file)
+    logging.debug(_('History file:               %s'), config.history_file)
+    logging.debug(_('Log file:                   %s'), config.log_file)
 
     # adapt application search path
     sys.path.insert(0, config.wxglade_path)
@@ -356,6 +357,8 @@ def init_stage2(use_gui):
             try:
                 import msw
                 msw.register_extensions(["wxg"], "wxGlade")
+                if not os.path.exists(config.rc_file):
+                    config.inform_screen_reader = msw.check_for_screen_reader()
             except ImportError:
                 pass
 
