@@ -1,11 +1,11 @@
 """
 Parsers used to load an app and to generate the code from a XML file.
 
-See L{wcodegen.taghandler} for custom tag handler base classes.
+See wcodegen.taghandler for custom tag handler base classes.
 
 @copyright: 2002-2007 Alberto Griggio
 @copyright: 2016 Carsten Grohmann
-@copyright: 2016-2018 Dietmar Schwertberger
+@copyright: 2016-2020 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
@@ -15,11 +15,7 @@ from xml.sax.handler import ContentHandler
 
 import time
 
-import common, config, compat
-import edit_sizers
-
-if config.use_gui:
-    import wx
+import common, config
 
 
 class XmlParsingError(SAXException):
@@ -36,7 +32,6 @@ class XmlParser(ContentHandler):
     "'abstract' base class of the parsers used to load an app and to generate the code"
 
     def __init__(self):
-        self._logger = logging.getLogger(self.__class__.__name__)
         self._objects = Stack()      # Stack of 'alive' objects
         self._curr_prop = None       # Name of the current property
         self._curr_prop_val = []     # Value of the current property; strings, to be joined
@@ -45,6 +40,7 @@ class XmlParser(ContentHandler):
         self.parser = make_parser()
         self.parser.setContentHandler(self)
         self.locator = None # Document locator
+        self.index = None     # only used with ClipboardXmlWidgetBuilder
 
     def parse(self, source):
         ## Permanent workaround for Python bug "Sax parser crashes if given
@@ -80,7 +76,7 @@ class XmlParser(ContentHandler):
 
         res['encoding'] = self._get_encoding(attrs)
 
-        for_version = attrs.get('for_version', '%s.%s' % config.for_version)
+        for_version = attrs.get('for_version', '%s.%s' % config.for_version_min)
         for_version_tuple = tuple([int(t) for t in for_version.split('.')[:2]])
         if for_version_tuple < (2, 8):
             logging.warning( _('The loaded wxGlade designs are created for wxWidgets "%s", '
@@ -88,8 +84,8 @@ class XmlParser(ContentHandler):
                              for_version )
             logging.warning( _('The designs will be loaded and converted to wxWidgets "%s" partially. '
                                'Please check the designs carefully.'),
-                            '%s.%s' % config.for_version )
-            for_version = '%s.%s' % config.for_version
+                            '%s.%s' % config.for_version_min )
+            for_version = '%s.%s' % config.for_version_min
         res['for_version'] = for_version
 
         try:
@@ -166,7 +162,7 @@ class XmlParser(ContentHandler):
             try:
                 'a'.encode(encoding)
             except LookupError:
-                self._logger.warning( _('Unknown encoding "%s", fallback to default encoding "%s"'),
+                logging.warning( _('Unknown encoding "%s", fallback to default encoding "%s"'),
                                       encoding, config.default_encoding)
                 encoding = config.default_encoding
         return encoding
@@ -181,12 +177,17 @@ class XmlWidgetBuilder(XmlParser):
         self.input_file_version = input_file_version
         XmlParser.__init__(self)
 
+    def check_input_file_version(self, version):
+        # return True if file version is older
+        if not self.input_file_version: return True
+        return self.input_file_version[:len(version)] < version
+
     def startElement(self, name, attrs):
         if name == 'application':
             # get properties of the app
             self._appl_started = True
             attrs = self._process_app_attrs(attrs)
-            app = common.app_tree.app
+            app = common.root
             p = app.properties
 
             p["encoding"].set( attrs['encoding'] )
@@ -250,7 +251,7 @@ class XmlWidgetBuilder(XmlParser):
     def endElement(self, name):
         if name == 'application':
             self._appl_started = False
-            app = common.app_tree.app
+            app = common.root
             for key, value in self._delayed_app_properties.items():
                 app.properties[key].set( value )
             app.properties_changed( sorted(self._delayed_app_properties.keys()) )
@@ -259,21 +260,30 @@ class XmlWidgetBuilder(XmlParser):
             # remove last object from Stack
             obj = self._objects.pop()
             obj.notify_owner()
-            if obj.klass in ('sizeritem', 'sizerslot'):
+            if obj.IS_SIZERITEM or obj.IS_SLOT:
                 return
-            if obj.sizeritem:
+            if obj.sizeritem and obj.obj.parent.IS_SIZER:
+                # XXX just check whether obj.obj has these properties
                 obj.obj.copy_properties( obj.sizeritem, ("option","flag","border","span") )
+                obj.obj.properties["flag"]._check_value()
             obj.obj.on_load()
         else:
             # end of a property or error
-            # case 1: set _curr_prop value
+            prop = self._curr_prop
             data = "".join(self._curr_prop_val)
+            self._curr_prop = None
+            self._curr_prop_val = []
+            if prop in ("menubar", "toolbar", "statusbar"):
+                # case 0: ignore properties: menubar, toolbar, statusbar
+                return
+
             if data:
+                # case 1: set _curr_prop value
                 try:
                     handler = self.top().prop_handlers.top()
                     if not handler or handler.char_data(data):
                         # if char_data returned False, we don't have to call add_property
-                        self.top().add_property(self._curr_prop, data)
+                        self.top().add_property(prop, data)
                 except AttributeError:
                     pass
 
@@ -287,8 +297,6 @@ class XmlWidgetBuilder(XmlParser):
                     obj.prop_handlers.pop()
             except AttributeError:
                 pass
-            self._curr_prop = None
-            self._curr_prop_val = []
 
     def characters(self, data):
         if not data or data.isspace():
@@ -301,12 +309,12 @@ class XmlWidgetBuilder(XmlParser):
 class ProgressXmlWidgetBuilder(XmlWidgetBuilder):
     "Adds support for a progress dialog to the widget builder parser"
 
-    def __init__(self, *args, **kwds):
-        self.input_file = kwds.get('input_file')
+    def __init__(self, filename, input_file_version, input_file):
+        self.input_file = input_file
         if self.input_file:
-            del kwds['input_file']
             self.size = len(self.input_file.readlines())
             self.input_file.seek(0)
+            import wx
             self.progress = wx.ProgressDialog( _("Loading..."), _("Please wait while loading the app"), 20 )
             self.step = 4
             self.i = 1
@@ -314,7 +322,7 @@ class ProgressXmlWidgetBuilder(XmlWidgetBuilder):
         else:
             self.size = 0
             self.progress = None
-        XmlWidgetBuilder.__init__(self)
+        XmlWidgetBuilder.__init__(self, filename, input_file_version)
 
     def endElement(self, name):
         if self.progress:
@@ -364,66 +372,65 @@ class ClipboardXmlWidgetBuilder(XmlWidgetBuilder):
         they keep info about the destination of the hierarchy of widgets (i.e. the target of the 'paste' command)
       - The first widget built must be hidden and shown again at the end of the operation"""
 
-    def __init__(self, parent, sizer, pos, proportion, span, flag, border):
+    def __init__(self, parent, index, proportion, span, flag, border):
         XmlWidgetBuilder.__init__(self)
         self._renamed = {}
-        if parent is not None:
-            self.parent_node = parent.node
+        self._object_counter = 0
+        self.parent = parent
+        self.index = index
+        if not parent:
+            # e.g. a frame is pasted: update with the top level names
+            self.have_names = set(child.name for child in common.root.children)
         else:
-            # e.g. a frame is pasted
-            self.parent_node = None
-            self.have_names = set()
-            # update with the top level names
-            for node in common.app_tree.names.keys():
-                self.have_names.add( node.widget.name )
+            self.have_names = set(parent.toplevel_parent.names)
 
         class XmlClipboardObject(object):
             def __init__(self, **kwds):
+                self.IS_SIZER = self.IS_WINDOW = False
                 self.__dict__.update(kwds)
             def notify_owner(self):
                 pass
 
         # fake parent window object
         fake_parent = XmlClipboardObject(obj=parent, parent=parent)
-        if parent and parent.is_sizer:
-            fake_parent.in_sizers = True
-            fake_parent.in_windows = False
+        if parent and parent.IS_SIZER:
+            fake_parent.IS_SIZER = True
         else:
-            fake_parent.in_sizers = False
-            fake_parent.in_windows = True
+            fake_parent.IS_WINDOW = True
 
         self._objects.push(fake_parent)
 
         # fake sizer object
-        if sizer:
+        if parent and parent.CHILDREN!=1:
+            sizer = parent
             fake_sizer = XmlClipboardObject(obj=sizer, parent=parent)
-            fake_sizer.in_windows = False
-            fake_sizer.in_sizers = True
+            fake_sizer.IS_SIZER = True
             sizeritem = Sizeritem()
             sizeritem.properties["proportion"].set(proportion)
             sizeritem.properties["span"].set(span)
             sizeritem.properties["flag"].set(flag)
             sizeritem.properties["border"].set(border)
-            sizeritem.properties["pos"].set(pos)
+            sizeritem.index = index
             # fake sizer item
             fake_sizeritem = XmlClipboardObject(obj=sizeritem, parent=parent)
-            fake_sizeritem.in_windows = False
-            fake_sizeritem.in_sizers = False
 
             self._objects.push(fake_sizer)
             self._objects.push(fake_sizeritem)
 
-
         self.depth_level = 0
         self._appl_started = True  # no application tag when parsing from the clipboard
 
     def _get_new_name(self, oldname):
+        # when pasting, check whether the name is free and if not get a new unique one
+        if not oldname in self.have_names:
+            self.have_names.add(oldname)
+            return oldname
         if self._renamed:
             # e.g. if notebook_1 was renamed to notebook_2, try notebook_1_panel_1 -> notebook_2_panel_1 first
             for old,new in self._renamed.items():
                 if oldname.startswith(old):
                     newname = new + oldname[len(old):]
-                    if not common.app_tree.has_name(newname, node=self.parent_node):
+                    if not newname in self.have_names:
                         return newname
 
         newname = oldname
@@ -431,13 +438,13 @@ class ClipboardXmlWidgetBuilder(XmlWidgetBuilder):
             # if the old name ends with an underscore and a number, just increase the number
             try:
                 template, i = oldname.rsplit("_", 1)
-                #i = int(i) + 1
                 i = int(i)
                 template = template + '_%s'
-                if self.parent_node is not None:
-                    while common.app_tree.has_name(newname, node=self.parent_node):
+                if self.parent is not None:
+                    while newname in self.have_names:
                         newname = template%i
                         i += 1
+                    self.have_names.add(newname)
                     return newname
                 # top level, e.g. a new frame is pasted
                 while newname in self.have_names:
@@ -448,15 +455,17 @@ class ClipboardXmlWidgetBuilder(XmlWidgetBuilder):
             except:
                 pass
 
-        # add _copy to the old name
+        # add _copy or _copy_N to the old name
+        if oldname.endswith('_copy'): oldname = oldname[:-5]
         i = 0
-        if self.parent_node is not None:
-            while common.app_tree.has_name(newname, node=self.parent_node):
+        if self.parent is not None:
+            while newname in self.have_names:
                 if not i:
                     newname = '%s_copy' % oldname
                 else:
                     newname = '%s_copy_%s' % (oldname, i)
                 i += 1
+            self.have_names.add(newname)
             return newname
 
         # top level, e.g. a new frame is pasted
@@ -471,13 +480,10 @@ class ClipboardXmlWidgetBuilder(XmlWidgetBuilder):
 
     def startElement(self, name, attrs):
         renamed = None
-        if name == 'object' and 'name' in attrs:
+        if name == 'object' and 'name' in attrs and common.widget_classes[attrs['base']].IS_NAMED:
             # generate a unique name for the copy
             oldname = str(attrs['name'])
-            if not common.app_tree.has_name(oldname, node=self.parent_node):
-                newname = oldname
-            else:
-                newname = self._get_new_name(oldname)
+            newname = self._get_new_name(oldname)
             attrs = _own_dict(attrs)
             attrs['name'] = newname
             if newname!=oldname:
@@ -492,26 +498,11 @@ class ClipboardXmlWidgetBuilder(XmlWidgetBuilder):
                 try:
                     self.top_obj = self.top().obj
                 except AttributeError:
-                    self._logger.exception( _('Exception caused by obj: %s'), self.top_obj )
+                    logging.exception( _('Exception caused by obj: %s'), self.top_obj )
             self.depth_level += 1
+            self._object_counter += 1
 
     def endElement(self, name):
-        if name == 'object':
-            obj = self.top()
-            self.depth_level -= 1
-            if not self.depth_level:
-                common.app_tree.auto_expand = True
-                import misc
-                try:
-                    # show the first object and update its layout
-                    #if self.top_obj.node.parent.widget.is_visible():
-                    #    common.app_tree.show_widget(self.top_obj.node)
-                    if self.top_obj.node.parent.widget.widget:
-                        self.top_obj.create_widgets()
-                except AttributeError:
-                    self._logger.exception( _('Exception caused by obj: %s'), self.top_obj )
-                misc.set_focused_widget(self.top_obj)
-
         if name == 'label':
             # if e.g. a button_1 is copied to button_2, also change the label if it was "button_1"
             obj = self.top()
@@ -523,103 +514,163 @@ class ClipboardXmlWidgetBuilder(XmlWidgetBuilder):
 
         XmlWidgetBuilder.endElement(self, name)
 
+        if name!="object": return
+        self.depth_level -= 1
+
+        if not self.depth_level:
+            if self.parent:
+                self.parent.on_load(child=self.top_obj)  # e.g. a GridBagSizer needs to check overlapped slots
+            common.app_tree.auto_expand = True
+            try:
+                if self.parent and self.parent.widget:
+                    self.top_obj.create()
+            except AttributeError:
+                logging.exception( _('Exception caused by obj: %s'), self.top_obj )
+
+
+class XMLAttrs(dict):
+    # raises XmlParsingError instead of KeyError
+    def __getitem__(self, key):
+        try:
+            return dict.__getitem__(self, key)
+        except KeyError:
+            if 'name' in self:
+                raise XmlParsingError( _("attribute %s missing from object '%s'")%(key, self['name']) )
+            raise XmlParsingError( _("attribute %s missing from object")%key )
+
 
 class XmlWidgetObject(object):
     "A class to encapsulate widget attributes read from a XML file, to store them until the widget can be created"
 
     def __init__(self, attrs, parser):
-        # initialise instance logger
-        self._logger = logging.getLogger(self.__class__.__name__)
-
-        attrs.input_file_version = parser.input_file_version  # for handling backwards compatibility on loading
+        attrs = XMLAttrs(attrs)
 
         self.prop_handlers = Stack()  # a stack of custom handler functions to set properties of this object
-        self.parser = parser
-        self.in_sizers = False   # if True, the widget is     a sizer, opposite of 'in_windows'
-        self.in_windows = False  # if True, the widget is not a sizer, opposite of 'in_sizers'
 
         self._properties_added = []
-        try:
-            base = attrs.get('base', None)
-            self.klass = attrs['class']
-        except KeyError:
-            raise XmlParsingError(_("'object' items must have a 'class' attribute"))
+        base = attrs.get('base', None)
+        klass = attrs['class']
 
         # find sizeritem, sizer, parent window
         sizeritem = sizer = parent = None
-        stack = self.parser._objects[:]
+        stack = parser._objects[:]
         top = stack.pop(-1) if stack else None
 
         if top and isinstance(top.obj, Sizeritem):
             sizeritem = top.obj
             top = stack.pop(-1)
-        if top and top.in_sizers:
+
+        if top and top.IS_SIZER:
             sizer = top.obj
             top = stack.pop(-1)
-        if top and top.in_windows:
+        if top and top.IS_WINDOW:
             parent = top.obj
+
         while parent is None and stack:
             top = stack.pop()
-            if top.in_windows: parent = top.obj
+            if top.IS_WINDOW: parent = top.obj
 
+        if parent is None:
+            parent = common.root
+
+        self.IS_SIZER = self.IS_WINDOW = self.IS_SLOT = self.IS_SIZERITEM = False
         if base is not None:
             # if base is not None, the object is a widget (or sizer), and not a sizeritem
-            self.sizeritem = sizeritem
+            self.sizeritem = sizeritem  # the properties will be copied later in endElement
 
-            pos = getattr(sizeritem, 'pos', None)
-            # XXX change this: don't use pos here at all; either we're just appending to the end or filling virtual sizers acc. to pagenames
-            #if pos is None and parent and hasattr(parent, 'virtual_sizer') and parent.virtual_sizer:
-            if pos is None and parent and hasattr(parent, 'virtual_sizer') and parent.virtual_sizer:
-                # virtual sizers don't use sizeritem objects around their items in XML; the index is found from the name
-                sizer = parent.virtual_sizer
-                sizer.node = parent.node
-                sizeritem = Sizeritem()
-                pos_ = sizer.get_itempos(attrs)
-                # XXXinsert empty slots before, if required
-                if pos_: pos = pos_  # when loading from a file, the names are set already and pos_ is not None
+            index = getattr(sizeritem, 'index', None)
+            if index is None and hasattr(parent, "get_itempos"):
+                # splitters and notebooks don't use sizeritems around their items in XML; pos is found from the name
+                index = parent.get_itempos(attrs)
+            elif not sizeritem and not stack:
+                index = parser.index
 
             # build the widget
-            if pos is not None: pos = int(pos)
             builder = common.widgets_from_xml.get(base, None)
             if builder is None: raise XmlParsingError("Widget '%s' not supported."%base)
-            self.obj = builder(attrs, parent, sizer, sizeritem, pos)
-            p = self.obj.properties.get("class")
-            if p:
-                p.set(self.klass)
-                common.app_tree.refresh(self.obj.node)
+            
+            self.obj = builder(parser, base, attrs["name"], sizer or parent, index)
+            self.set_class_attributes(parser, attrs) # set 'class' and 'instance_class' properties, if applicable
 
-            if isinstance(self.obj, edit_sizers.SizerBase):
-                self.in_sizers = True
-            else:
-                self.in_windows = True
+            self.IS_SIZER = self.obj.IS_SIZER
+            self.IS_WINDOW = self.obj.IS_WINDOW
 
-        elif self.klass == 'sizeritem':
+        elif klass == 'sizeritem':
             self.obj = Sizeritem()
             self.parent = parent
+            self.IS_SIZERITEM = True
 
-        elif self.klass == 'sizerslot':
+        elif klass == 'sizerslot':
             assert sizer is not None, _("malformed wxg file: slots can only be inside sizers!")
+            self.obj = None
+            self.IS_SLOT = True
             sizer._add_slot(loading=True)
-            sizer.layout()
 
         # push the object on the _objects stack
-        self.parser._objects.push(self)
+        parser._objects.push(self)
+
+    def set_class_attributes(self, parser, attrs):
+        # old files: no 'instance_class' attribute, 'class' is 'multi purpose'
+        CLASS = self.obj.__class__
+        class_p = self.obj.properties.get("class")
+
+        class_v = attrs.get("class") or None
+        IS_BASE = class_v==CLASS.WX_CLASS or (CLASS.WX_CLASSES and class_v in CLASS.WX_CLASSES)
+
+        instance_class = attrs.get("instance_class")
+
+        if parser.check_input_file_version( (0,9,9) ):
+            # handle backwards compatibility
+            if class_p:
+                if class_p.deactivated is not None:  # i.e. 'class' property is not mandatory / ClassPropertyD
+                    if IS_BASE:
+                        class_v = None
+            elif class_v:
+                if not IS_BASE or CLASS.WX_CLASS=="CustomWidget":
+                    instance_class = class_v
+                class_v = None
+            if attrs.get("no_custom_class", None) in ('1',1):
+                if class_v:
+                    instance_class = class_v
+                    class_v = None
+        else:
+            # current file format: 'class' is always written
+            # if the object does not have a 'class' property at all, the value is the instance class
+            # if it has a 'class' property, the 'instance_class' is written if required
+            if class_p:
+                if class_p.deactivated is not None and IS_BASE:
+                    class_v = None
+            else:
+                if not IS_BASE and instance_class is None:
+                    instance_class = class_v
+                class_v = None
+
+        # update self.obj properties
+        modified = []
+        if class_v:
+            class_p.set( class_v, activate=True )
+            modified.append("class")
+        if instance_class:
+            self.obj.properties["instance_class"].set( instance_class, activate=True )
+            modified.append("instance_class")
+        if modified:
+            self.obj.properties_changed(modified)
 
     def add_property(self, name, val):
         """adds a property to this widget. This method is not called if there
         was a custom handler for this property, and its char_data method returned False"""
+        if name=="no_custom_class": return  # this was also stored as attribute
         if name == 'pos':  # sanity check, this shouldn't happen...
-            self._logger.debug('add_property(name=pos)')
+            logging.debug('add_property(name=pos)')
             return
         try:
             prop = self.obj.properties[name]
         except KeyError:
             # unknown property for this object; issue a warning and ignore the property
             if config.debugging: raise
-            self._logger.error( _("WARNING: Property '%s' not supported by this object ('%s') "), name, self.obj )
+            logging.error( _("WARNING: Property '%s' not supported by this object ('%s') "), name, self.obj )
             return
         prop.load(val, activate=True)
-        #self.obj.properties_changed([name])
         self._properties_added.append(name)
 
     def notify_owner(self):
@@ -645,11 +696,13 @@ import new_properties as np
 
 class Sizeritem(np.PropertyOwner):
     "temporarily represents a child of a sizer"
-    SIZER_PROPERTIES = ["pos","proportion","span","border","flag"]
+    SIZER_PROPERTIES = ["proportion","span","border","flag"]
     def __init__(self):
         np.PropertyOwner.__init__(self)
         self.proportion = np.LayoutProportionProperty(0)
         self.span = np.LayoutSpanProperty((1,1))
         self.border = np.SpinProperty(0)
-        self.flag = np.ManagedFlags(None)
-        self.pos = np.SpinProperty(None)
+        self.flag = np.ManagedFlags(None, name="sizeritem_flags")
+
+    def on_load(self, child=None):
+        pass
\ No newline at end of file
